# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Master view for Plants
"""

from webhelpers2.html import tags

from wuttaweb.forms.schema import WuttaDictEnum
from wuttaweb.util import get_form_data

from wuttafarm.db.model import PlantType, Season, PlantAsset
from wuttafarm.web.views.assets import AssetTypeMasterView, AssetMasterView
from wuttafarm.web.forms.schema import PlantTypeRefs, SeasonRefs
from wuttafarm.web.forms.widgets import ImageWidget
from wuttafarm.web.util import get_farmos_client_for_user


class PlantTypeView(AssetTypeMasterView):
    """
    Master view for Plant Types
    """

    model_class = PlantType
    route_prefix = "plant_types"
    url_prefix = "/plant-types"

    farmos_entity_type = "taxonomy_term"
    farmos_bundle = "plant_type"
    farmos_refurl_path = "/admin/structure/taxonomy/manage/plant_type/overview"

    grid_columns = [
        "name",
        "description",
    ]

    sort_defaults = "name"

    filter_defaults = {
        "name": {"active": True, "verb": "contains"},
    }

    form_fields = [
        "name",
        "description",
        "drupal_id",
        "farmos_uuid",
    ]

    has_rows = True
    row_model_class = PlantAsset
    rows_viewable = True

    row_grid_columns = [
        "asset_name",
        "archived",
    ]

    rows_sort_defaults = "asset_name"

    def configure_grid(self, grid):
        g = grid
        super().configure_grid(g)

        # name
        g.set_link("name")

    def get_farmos_url(self, plant_type):
        return self.app.get_farmos_url(f"/taxonomy/term/{plant_type.drupal_id}")

    def get_xref_buttons(self, plant_type):
        buttons = super().get_xref_buttons(plant_type)

        if plant_type.farmos_uuid:
            buttons.append(
                self.make_button(
                    "View farmOS record",
                    primary=True,
                    url=self.request.route_url(
                        "farmos_plant_types.view", uuid=plant_type.farmos_uuid
                    ),
                    icon_left="eye",
                )
            )

        return buttons

    def delete(self):
        plant_type = self.get_instance()

        if plant_type._plant_assets:
            self.request.session.flash(
                "Cannot delete plant type which is still referenced by plant assets.",
                "warning",
            )
            url = self.get_action_url("view", plant_type)
            return self.redirect(self.request.get_referrer(default=url))

        return super().delete()

    def get_row_grid_data(self, plant_type):
        model = self.app.model
        session = self.Session()
        return (
            session.query(model.PlantAsset)
            .join(model.Asset)
            .outerjoin(model.PlantAssetPlantType)
            .filter(model.PlantAssetPlantType.plant_type == plant_type)
        )

    def configure_row_grid(self, grid):
        g = grid
        super().configure_row_grid(g)
        model = self.app.model

        # asset_name
        g.set_link("asset_name")
        g.set_sorter("asset_name", model.Asset.asset_name)
        g.set_filter("asset_name", model.Asset.asset_name)

        # archived
        g.set_renderer("archived", "boolean")
        g.set_sorter("archived", model.Asset.archived)
        g.set_filter("archived", model.Asset.archived)

    def get_row_action_url_view(self, plant, i):
        return self.request.route_url("plant_assets.view", uuid=plant.uuid)

    def ajax_create(self):
        """
        AJAX view to create a new plant type.
        """
        model = self.app.model
        session = self.Session()
        data = get_form_data(self.request)

        name = data.get("name")
        if not name:
            return {"error": "Name is required"}

        plant_type = model.PlantType(name=name)
        session.add(plant_type)
        session.flush()

        if self.app.is_farmos_mirror():
            client = get_farmos_client_for_user(self.request)
            self.app.auto_sync_to_farmos(plant_type, client=client)

        return {
            "uuid": plant_type.uuid.hex,
            "name": plant_type.name,
            "farmos_uuid": plant_type.farmos_uuid.hex,
            "drupal_id": plant_type.drupal_id,
        }

    @classmethod
    def defaults(cls, config):
        """ """
        cls._defaults(config)
        cls._plant_type_defaults(config)

    @classmethod
    def _plant_type_defaults(cls, config):
        route_prefix = cls.get_route_prefix()
        permission_prefix = cls.get_permission_prefix()
        url_prefix = cls.get_url_prefix()

        # ajax_create
        config.add_route(f"{route_prefix}.ajax_create", f"{url_prefix}/ajax/new")
        config.add_view(
            cls,
            attr="ajax_create",
            route_name=f"{route_prefix}.ajax_create",
            permission=f"{permission_prefix}.create",
            renderer="json",
        )


class SeasonView(AssetTypeMasterView):
    """
    Master view for Seasons
    """

    model_class = Season
    route_prefix = "seasons"
    url_prefix = "/seasons"

    farmos_entity_type = "taxonomy_term"
    farmos_bundle = "season"
    farmos_refurl_path = "/admin/structure/taxonomy/manage/season/overview"

    grid_columns = [
        "name",
        "description",
    ]

    sort_defaults = "name"

    filter_defaults = {
        "name": {"active": True, "verb": "contains"},
    }

    form_fields = [
        "name",
        "description",
        "drupal_id",
        "farmos_uuid",
    ]

    has_rows = True
    row_model_class = PlantAsset
    rows_viewable = True

    row_grid_columns = [
        "asset_name",
        "archived",
    ]

    rows_sort_defaults = "asset_name"

    def configure_grid(self, grid):
        g = grid
        super().configure_grid(g)

        # name
        g.set_link("name")

    def get_farmos_url(self, season):
        return self.app.get_farmos_url(f"/taxonomy/term/{season.drupal_id}")

    def get_xref_buttons(self, season):
        buttons = super().get_xref_buttons(season)

        if season.farmos_uuid:
            buttons.append(
                self.make_button(
                    "View farmOS record",
                    primary=True,
                    url=self.request.route_url(
                        "farmos_seasons.view", uuid=season.farmos_uuid
                    ),
                    icon_left="eye",
                )
            )

        return buttons

    def delete(self):
        season = self.get_instance()

        if season._plant_assets:
            self.request.session.flash(
                "Cannot delete season which is still referenced by plant assets.",
                "warning",
            )
            url = self.get_action_url("view", season)
            return self.redirect(self.request.get_referrer(default=url))

        return super().delete()

    def get_row_grid_data(self, season):
        model = self.app.model
        session = self.Session()
        return (
            session.query(model.PlantAsset)
            .join(model.Asset)
            .outerjoin(model.PlantAssetSeason)
            .filter(model.PlantAssetSeason.season == season)
        )

    def configure_row_grid(self, grid):
        g = grid
        super().configure_row_grid(g)
        model = self.app.model

        # asset_name
        g.set_link("asset_name")
        g.set_sorter("asset_name", model.Asset.asset_name)
        g.set_filter("asset_name", model.Asset.asset_name)

        # archived
        g.set_renderer("archived", "boolean")
        g.set_sorter("archived", model.Asset.archived)
        g.set_filter("archived", model.Asset.archived)

    def get_row_action_url_view(self, plant, i):
        return self.request.route_url("plant_assets.view", uuid=plant.uuid)

    def ajax_create(self):
        """
        AJAX view to create a new season.
        """
        model = self.app.model
        session = self.Session()
        data = get_form_data(self.request)

        name = data.get("name")
        if not name:
            return {"error": "Name is required"}

        season = model.Season(name=name)
        session.add(season)
        session.flush()

        if self.app.is_farmos_mirror():
            client = get_farmos_client_for_user(self.request)
            self.app.auto_sync_to_farmos(season, client=client)

        return {
            "uuid": season.uuid.hex,
            "name": season.name,
            "farmos_uuid": season.farmos_uuid.hex,
            "drupal_id": season.drupal_id,
        }

    @classmethod
    def defaults(cls, config):
        """ """
        cls._defaults(config)
        cls._season_defaults(config)

    @classmethod
    def _season_defaults(cls, config):
        route_prefix = cls.get_route_prefix()
        permission_prefix = cls.get_permission_prefix()
        url_prefix = cls.get_url_prefix()

        # ajax_create
        config.add_route(f"{route_prefix}.ajax_create", f"{url_prefix}/ajax/new")
        config.add_view(
            cls,
            attr="ajax_create",
            route_name=f"{route_prefix}.ajax_create",
            permission=f"{permission_prefix}.create",
            renderer="json",
        )


class PlantAssetView(AssetMasterView):
    """
    Master view for Plant Assets
    """

    model_class = PlantAsset
    route_prefix = "plant_assets"
    url_prefix = "/assets/plant"

    farmos_bundle = "plant"
    farmos_refurl_path = "/assets/plant"

    labels = {
        "plant_types": "Crop/Variety",
        "seasons": "Season",
    }

    grid_columns = [
        "thumbnail",
        "drupal_id",
        "asset_name",
        "plant_types",
        "season",
        "archived",
    ]

    def configure_grid(self, grid):
        g = grid
        super().configure_grid(g)

        # plant_types
        g.set_renderer("plant_types", self.render_plant_types_for_grid)

    def render_plant_types_for_grid(self, plant, field, value):
        plant_types = plant._plant_types

        if self.farmos_style_grid_links:
            links = []
            for plant_type in plant_types:
                plant_type = plant_type.plant_type
                url = self.request.route_url("plant_types.view", uuid=plant_type.uuid)
                links.append(tags.link_to(str(plant_type), url))
            return ", ".join(links)

        return ", ".join([str(pt.plant_type) for pt in plant_types])

    def configure_form(self, form):
        f = form
        super().configure_form(f)
        enum = self.app.enum
        plant = f.model_instance

        # plant_types
        f.fields.insert_after("asset_name", "plant_types")
        f.set_node("plant_types", PlantTypeRefs(self.request))
        if not self.creating:
            # nb. must explcitly declare value for non-standard field
            f.set_default("plant_types", [pt.uuid for pt in plant.plant_types])

        # season
        f.fields.insert_after("plant_types", "seasons")
        f.set_node("seasons", SeasonRefs(self.request))
        f.set_required("seasons", False)
        if not self.creating:
            # nb. must explcitly declare value for non-standard field
            f.set_default("seasons", plant.seasons)

    def objectify(self, form):
        plant = super().objectify(form)
        data = form.validated

        self.set_plant_types(plant, data["plant_types"])
        self.set_seasons(plant, data["seasons"])

        return plant

    def set_plant_types(self, plant, desired):
        model = self.app.model
        session = self.Session()
        current = [pt.uuid for pt in plant.plant_types]

        for uuid in desired:
            if uuid not in current:
                plant_type = session.get(model.PlantType, uuid)
                assert plant_type
                plant.plant_types.append(plant_type)

        for uuid in current:
            if uuid not in desired:
                plant_type = session.get(model.PlantType, uuid)
                assert plant_type
                plant.plant_types.remove(plant_type)

    def set_seasons(self, plant, desired):
        model = self.app.model
        session = self.Session()
        current = [s.uuid for s in plant.seasons]

        for uuid in desired:
            if uuid not in current:
                season = session.get(model.Season, uuid)
                assert season
                plant.seasons.append(season)

        for uuid in current:
            if uuid not in desired:
                season = session.get(model.Season, uuid)
                assert season
                plant.seasons.remove(season)


def defaults(config, **kwargs):
    base = globals()

    PlantTypeView = kwargs.get("PlantTypeView", base["PlantTypeView"])
    PlantTypeView.defaults(config)

    SeasonView = kwargs.get("SeasonView", base["SeasonView"])
    SeasonView.defaults(config)

    PlantAssetView = kwargs.get("PlantAssetView", base["PlantAssetView"])
    PlantAssetView.defaults(config)


def includeme(config):
    defaults(config)



"""add Measure.ordinal

Revision ID: 82a497e30a97
Revises: c5183b781d34
Create Date: 2026-03-08 13:15:36.917747

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import wuttjamaican.db.util


# revision identifiers, used by Alembic.
revision: str = "82a497e30a97"
down_revision: Union[str, None] = "c5183b781d34"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:

    # measure
    op.add_column("measure", sa.Column("ordinal", sa.Integer(), nullable=True))
    op.add_column(
        "measure_version",
        sa.Column("ordinal", sa.Integer(), autoincrement=False, nullable=True),
    )


def downgrade() -> None:

    # measure
    op.drop_column("measure_version", "ordinal")
    op.drop_column("measure", "ordinal")



"""add Land Types

Revision ID: 9f2243df9566
Revises: cf3f8f46d8bc
Create Date: 2026-02-10 19:10:02.851756

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import wuttjamaican.db.util


# revision identifiers, used by Alembic.
revision: str = "9f2243df9566"
down_revision: Union[str, None] = "cf3f8f46d8bc"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:

    # land_type
    op.create_table(
        "land_type",
        sa.Column("uuid", wuttjamaican.db.util.UUID(), nullable=False),
        sa.Column("name", sa.String(length=100), nullable=False),
        sa.Column("farmos_uuid", wuttjamaican.db.util.UUID(), nullable=True),
        sa.Column("drupal_id", sa.String(length=50), nullable=True),
        sa.PrimaryKeyConstraint("uuid", name=op.f("pk_land_type")),
        sa.UniqueConstraint("drupal_id", name=op.f("uq_land_type_drupal_id")),
        sa.UniqueConstraint("farmos_uuid", name=op.f("uq_land_type_farmos_uuid")),
        sa.UniqueConstraint("name", name=op.f("uq_land_type_name")),
    )
    op.create_table(
        "land_type_version",
        sa.Column(
            "uuid", wuttjamaican.db.util.UUID(), autoincrement=False, nullable=False
        ),
        sa.Column("name", sa.String(length=100), autoincrement=False, nullable=True),
        sa.Column(
            "farmos_uuid",
            wuttjamaican.db.util.UUID(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "drupal_id", sa.String(length=50), autoincrement=False, nullable=True
        ),
        sa.Column(
            "transaction_id", sa.BigInteger(), autoincrement=False, nullable=False
        ),
        sa.Column("end_transaction_id", sa.BigInteger(), nullable=True),
        sa.Column("operation_type", sa.SmallInteger(), nullable=False),
        sa.PrimaryKeyConstraint(
            "uuid", "transaction_id", name=op.f("pk_land_type_version")
        ),
    )
    op.create_index(
        op.f("ix_land_type_version_end_transaction_id"),
        "land_type_version",
        ["end_transaction_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_land_type_version_operation_type"),
        "land_type_version",
        ["operation_type"],
        unique=False,
    )
    op.create_index(
        "ix_land_type_version_pk_transaction_id",
        "land_type_version",
        ["uuid", sa.literal_column("transaction_id DESC")],
        unique=False,
    )
    op.create_index(
        "ix_land_type_version_pk_validity",
        "land_type_version",
        ["uuid", "transaction_id", "end_transaction_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_land_type_version_transaction_id"),
        "land_type_version",
        ["transaction_id"],
        unique=False,
    )


def downgrade() -> None:

    # land_type
    op.drop_index(
        op.f("ix_land_type_version_transaction_id"), table_name="land_type_version"
    )
    op.drop_index("ix_land_type_version_pk_validity", table_name="land_type_version")
    op.drop_index(
        "ix_land_type_version_pk_transaction_id", table_name="land_type_version"
    )
    op.drop_index(
        op.f("ix_land_type_version_operation_type"), table_name="land_type_version"
    )
    op.drop_index(
        op.f("ix_land_type_version_end_transaction_id"), table_name="land_type_version"
    )
    op.drop_table("land_type_version")
    op.drop_table("land_type")



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Model definition for Quantities
"""

import sqlalchemy as sa
from sqlalchemy import orm
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.ext.associationproxy import association_proxy

from wuttjamaican.db import model


class QuantityType(model.Base):
    """
    Represents an "quantity type" from farmOS
    """

    __tablename__ = "quantity_type"
    __versioned__ = {}
    __wutta_hint__ = {
        "model_title": "Quantity Type",
        "model_title_plural": "Quantity Types",
    }

    uuid = model.uuid_column()

    name = sa.Column(
        sa.String(length=100),
        nullable=False,
        unique=True,
        doc="""
        Name of the quantity type.
        """,
    )

    description = sa.Column(
        sa.String(length=255),
        nullable=True,
        doc="""
        Description for the quantity type.
        """,
    )

    farmos_uuid = sa.Column(
        model.UUID(),
        nullable=True,
        unique=True,
        doc="""
        UUID for the quantity type within farmOS.
        """,
    )

    drupal_id = sa.Column(
        sa.String(length=50),
        nullable=True,
        unique=True,
        doc="""
        Drupal internal ID for the quantity type.
        """,
    )

    def __str__(self):
        return self.name or ""


class Quantity(model.Base):
    """
    Represents a base quantity record from farmOS
    """

    __tablename__ = "quantity"
    __versioned__ = {}
    __wutta_hint__ = {
        "model_title": "Quantity",
        "model_title_plural": "All Quantities",
    }

    uuid = model.uuid_column()

    quantity_type_id = sa.Column(
        sa.String(length=50),
        sa.ForeignKey("quantity_type.drupal_id"),
        nullable=False,
    )

    quantity_type = orm.relationship(QuantityType)

    measure_id = sa.Column(
        sa.String(length=20),
        sa.ForeignKey("measure.drupal_id"),
        nullable=False,
        doc="""
        Measure for the quantity.
        """,
    )

    measure = orm.relationship("Measure")

    value_numerator = sa.Column(
        sa.Integer(),
        nullable=False,
        doc="""
        Numerator for the quantity value.
        """,
    )

    value_denominator = sa.Column(
        sa.Integer(),
        nullable=False,
        doc="""
        Denominator for the quantity value.
        """,
    )

    units_uuid = model.uuid_fk_column("unit.uuid", nullable=False)
    units = orm.relationship("Unit")

    label = sa.Column(
        sa.String(length=255),
        nullable=True,
        doc="""
        Optional label for the quantity.
        """,
    )

    farmos_uuid = sa.Column(
        model.UUID(),
        nullable=True,
        unique=True,
        doc="""
        UUID for the quantity within farmOS.
        """,
    )

    drupal_id = sa.Column(
        sa.Integer(),
        nullable=True,
        unique=True,
        doc="""
        Drupal internal ID for the quantity.
        """,
    )

    _log = orm.relationship(
        "LogQuantity",
        uselist=False,
        cascade="all, delete-orphan",
        cascade_backrefs=False,
        back_populates="quantity",
    )

    def make_log_quantity(log):
        from wuttafarm.db.model import LogQuantity

        return LogQuantity(log=log)

    log = association_proxy(
        "_log",
        "log",
        creator=make_log_quantity,
    )

    def get_value_decimal(self):
        # TODO: should actually return a decimal here?
        return self.value_numerator / self.value_denominator

    def render_as_text(self, config=None):
        measure = str(self.measure or self.measure_id or "")
        value = self.get_value_decimal()
        if config:
            app = config.get_app()
            value = app.render_quantity(value)
        units = str(self.units or "")
        return f"( {measure} ) {value} {units}"

    def __str__(self):
        return self.render_as_text()


class QuantityMixin:

    uuid = model.uuid_fk_column("quantity.uuid", nullable=False, primary_key=True)

    @declared_attr
    def quantity(cls):
        return orm.relationship(
            Quantity,
            single_parent=True,
            cascade="all, delete-orphan",
            cascade_backrefs=False,
        )

    def get_value_decimal(self):
        return self.quantity.get_value_decimal()

    def render_as_text(self, config=None):
        return self.quantity.render_as_text(config)

    def __str__(self):
        return self.render_as_text()


def add_quantity_proxies(subclass):
    Quantity.make_proxy(subclass, "quantity", "farmos_uuid")
    Quantity.make_proxy(subclass, "quantity", "drupal_id")
    Quantity.make_proxy(subclass, "quantity", "quantity_type")
    Quantity.make_proxy(subclass, "quantity", "quantity_type_id")
    Quantity.make_proxy(subclass, "quantity", "measure")
    Quantity.make_proxy(subclass, "quantity", "measure_id")
    Quantity.make_proxy(subclass, "quantity", "value_numerator")
    Quantity.make_proxy(subclass, "quantity", "value_denominator")
    Quantity.make_proxy(subclass, "quantity", "value_decimal")
    Quantity.make_proxy(subclass, "quantity", "units_uuid")
    Quantity.make_proxy(subclass, "quantity", "units")
    Quantity.make_proxy(subclass, "quantity", "label")
    Quantity.make_proxy(subclass, "quantity", "log")


class StandardQuantity(QuantityMixin, model.Base):
    """
    Represents a Standard Quantity from farmOS
    """

    __tablename__ = "quantity_standard"
    __versioned__ = {}
    __wutta_hint__ = {
        "model_title": "Standard Quantity",
        "model_title_plural": "Standard Quantities",
        "farmos_quantity_type": "standard",
    }


add_quantity_proxies(StandardQuantity)


class MaterialQuantity(QuantityMixin, model.Base):
    """
    Represents a Material Quantity from farmOS
    """

    __tablename__ = "quantity_material"
    __versioned__ = {}
    __wutta_hint__ = {
        "model_title": "Material Quantity",
        "model_title_plural": "Material Quantities",
        "farmos_quantity_type": "material",
    }

    _material_types = orm.relationship(
        "MaterialQuantityMaterialType",
        cascade="all, delete-orphan",
        cascade_backrefs=False,
        back_populates="quantity",
    )

    material_types = association_proxy(
        "_material_types",
        "material_type",
        creator=lambda mtype: MaterialQuantityMaterialType(material_type=mtype),
    )

    def render_as_text(self, config=None):
        text = super().render_as_text(config)
        mtypes = ", ".join([str(mt) for mt in self.material_types])
        return f"{mtypes} {text}"


add_quantity_proxies(MaterialQuantity)


class MaterialQuantityMaterialType(model.Base):
    """
    Represents a "material quantity's material type relationship" from
    farmOS.
    """

    __tablename__ = "quantity_material_material_type"
    __versioned__ = {}

    uuid = model.uuid_column()

    quantity_uuid = model.uuid_fk_column("quantity_material.uuid", nullable=False)
    quantity = orm.relationship(
        MaterialQuantity,
        foreign_keys=quantity_uuid,
        back_populates="_material_types",
    )

    material_type_uuid = model.uuid_fk_column("material_type.uuid", nullable=False)
    material_type = orm.relationship(
        "MaterialType",
        foreign_keys=material_type_uuid,
        back_populates="_quantities",
    )



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
View for farmOS Land Assets
"""

import datetime

import colander

from wuttaweb.forms.schema import WuttaDateTime
from wuttaweb.forms.widgets import WuttaDateTimeWidget

from wuttafarm.web.views.farmos import FarmOSMasterView


class LandAssetView(FarmOSMasterView):
    """
    View for farmOS Land Assets
    """

    model_name = "farmos_land_asset"
    model_title = "farmOS Land Asset"
    model_title_plural = "farmOS Land Assets"

    route_prefix = "farmos_land_assets"
    url_prefix = "/farmOS/land"

    farmos_refurl_path = "/assets/land"

    grid_columns = [
        "name",
        "land_type",
        "is_fixed",
        "is_location",
        "archived",
        "changed",
    ]

    sort_defaults = "name"

    form_fields = [
        "name",
        "land_type",
        "is_fixed",
        "is_location",
        "archived",
        "notes",
        "created",
        "changed",
    ]

    def get_grid_data(self, columns=None, session=None):
        land_assets = self.farmos_client.resource.get("asset", "land")
        return [self.normalize_land_asset(l) for l in land_assets["data"]]

    def configure_grid(self, grid):
        g = grid
        super().configure_grid(g)

        # name
        g.set_link("name")
        g.set_searchable("name")

        # is_fixed
        g.set_renderer("is_fixed", "boolean")

        # is_location
        g.set_renderer("is_location", "boolean")

        # created
        g.set_renderer("created", "datetime")

        # changed
        g.set_renderer("changed", "datetime")

        # archived
        g.set_renderer("archived", "boolean")

    def get_instance(self):
        land_asset = self.farmos_client.resource.get_id(
            "asset", "land", self.request.matchdict["uuid"]
        )
        self.raw_json = land_asset
        return self.normalize_land_asset(land_asset["data"])

    def get_instance_title(self, land_asset):
        return land_asset["name"]

    def normalize_land_asset(self, land):

        if created := land["attributes"].get("created"):
            created = datetime.datetime.fromisoformat(created)
            created = self.app.localtime(created)

        if changed := land["attributes"].get("changed"):
            changed = datetime.datetime.fromisoformat(changed)
            changed = self.app.localtime(changed)

        if notes := land["attributes"]["notes"]:
            notes = notes["value"]

        if self.farmos_4x:
            archived = land["attributes"]["archived"]
        else:
            archived = land["attributes"]["status"] == "archived"

        return {
            "uuid": land["id"],
            "drupal_id": land["attributes"]["drupal_internal__id"],
            "name": land["attributes"]["name"],
            "land_type": land["attributes"]["land_type"],
            "created": created,
            "changed": changed,
            "is_fixed": land["attributes"]["is_fixed"],
            "is_location": land["attributes"]["is_location"],
            "archived": archived,
            "notes": notes or colander.null,
        }

    def configure_form(self, form):
        f = form
        super().configure_form(f)

        # is_fixed
        f.set_node("is_fixed", colander.Boolean())

        # is_location
        f.set_node("is_location", colander.Boolean())

        # notes
        f.set_widget("notes", "notes")

        # created
        f.set_node("created", WuttaDateTime())
        f.set_widget("created", WuttaDateTimeWidget(self.request))

        # changed
        f.set_node("changed", WuttaDateTime())
        f.set_widget("changed", WuttaDateTimeWidget(self.request))

        # archived
        f.set_node("archived", colander.Boolean())

    def get_xref_buttons(self, land):
        return [
            self.make_button(
                "View in farmOS",
                primary=True,
                url=self.app.get_farmos_url(f"/asset/{land['drupal_id']}"),
                target="_blank",
                icon_left="external-link-alt",
            ),
        ]

    def get_xref_buttons(self, land):
        model = self.app.model
        session = self.Session()

        buttons = [
            self.make_button(
                "View in farmOS",
                primary=True,
                url=self.app.get_farmos_url(f"/asset/{land['drupal_id']}"),
                target="_blank",
                icon_left="external-link-alt",
            ),
        ]

        if wf_land := (
            session.query(model.LandAsset)
            .filter(model.LandAsset.farmos_uuid == land["uuid"])
            .first()
        ):
            buttons.append(
                self.make_button(
                    f"View {self.app.get_title()} record",
                    primary=True,
                    url=self.request.route_url("land_assets.view", uuid=wf_land.uuid),
                    icon_left="eye",
                )
            )

        return buttons


def defaults(config, **kwargs):
    base = globals()

    LandAssetView = kwargs.get("LandAssetView", base["LandAssetView"])
    LandAssetView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Master view for Assets
"""

from collections import OrderedDict

from webhelpers2.html import tags

from wuttaweb.forms.schema import WuttaDictEnum
from wuttaweb.db import Session

from wuttafarm.web.views import WuttaFarmMasterView
from wuttafarm.db.model import Asset, Log
from wuttafarm.web.forms.schema import OwnerRefs, AssetRefs
from wuttafarm.web.forms.widgets import ImageWidget
from wuttafarm.util import get_log_type_enum
from wuttafarm.web.util import get_farmos_client_for_user


def get_asset_type_enum(config):
    app = config.get_app()
    model = app.model
    session = Session()
    asset_types = OrderedDict()
    query = session.query(model.AssetType).order_by(model.AssetType.name)
    for asset_type in query:
        asset_types[asset_type.drupal_id] = asset_type.name
    return asset_types


class AssetTypeMasterView(WuttaFarmMasterView):
    """
    Base class for "Asset Type" master views.

    A bit of a misnmer perhaps, this is *not* for the actual AssetType
    model, but rather the "secondary" types, e.g.  AnimalType,
    LandType etc.
    """


class AssetMasterView(WuttaFarmMasterView):
    """
    Base class for Asset master views
    """

    farmos_entity_type = "asset"

    labels = {
        "groups": "Group Membership",
    }

    sort_defaults = "asset_name"

    filter_defaults = {
        "asset_name": {"active": True, "verb": "contains"},
        "archived": {"active": True, "verb": "is_false"},
    }

    form_fields = [
        "asset_name",
        "parents",
        "notes",
        "asset_type",
        "owners",
        "is_location",
        "is_fixed",
        "locations",
        "groups",
        "archived",
        "drupal_id",
        "farmos_uuid",
        "thumbnail_url",
        "image_url",
        "thumbnail",
        "image",
    ]

    has_rows = True
    row_model_class = Log
    rows_viewable = True

    row_labels = {
        "message": "Log Name",
    }

    row_grid_columns = [
        "status",
        "drupal_id",
        "timestamp",
        "message",
        "log_type",
        "assets",
        "location",
        "quantity",
        "is_group_assignment",
    ]

    rows_sort_defaults = ("timestamp", "desc")

    def get_fallback_templates(self, template):
        templates = super().get_fallback_templates(template)

        if self.viewing:
            templates.insert(0, "/assets/master/view.mako")

        return templates

    def get_query(self, session=None):
        """ """
        model = self.app.model
        model_class = self.get_model_class()
        session = session or self.Session()
        query = session.query(model_class)
        if model_class is not model.Asset:
            query = query.join(model.Asset)
        return query

    def configure_grid(self, grid):
        g = grid
        super().configure_grid(g)
        model = self.app.model

        # thumbnail
        g.set_renderer("thumbnail", self.render_grid_thumbnail)
        g.set_label("thumbnail", "", column_only=True)
        g.set_centered("thumbnail")

        # drupal_id
        g.set_label("drupal_id", "ID", column_only=True)
        g.set_sorter("drupal_id", model.Asset.drupal_id)
        g.set_filter("drupal_id", model.Asset.drupal_id)

        # asset_name
        g.set_link("asset_name")
        g.set_sorter("asset_name", model.Asset.asset_name)
        g.set_filter("asset_name", model.Asset.asset_name)

        # parents
        g.set_renderer("parents", self.render_parents_for_grid)

        # groups
        g.set_renderer("groups", self.render_groups_for_grid)

        # owners
        g.set_label("owners", "Owner")
        g.set_renderer("owners", self.render_owners_for_grid)

        # locations
        g.set_label("locations", "Location")
        g.set_renderer("locations", self.render_locations_for_grid)

        # archived
        g.set_renderer("archived", "boolean")
        g.set_sorter("archived", model.Asset.archived)
        g.set_filter("archived", model.Asset.archived)

    def render_parents_for_grid(self, asset, field, value):

        if self.farmos_style_grid_links:
            links = []
            for parent in asset.parents:
                url = self.request.route_url(
                    f"{parent.asset_type}_assets.view", uuid=parent.uuid
                )
                links.append(tags.link_to(str(parent), url))
            return ", ".join(links)

        parents = [str(p.parent) for p in asset.parents]
        return ", ".join(parents)

    def render_owners_for_grid(self, asset, field, value):

        if self.farmos_style_grid_links:
            links = []
            for user in asset.owners:
                url = self.request.route_url("users.view", uuid=user.uuid)
                links.append(tags.link_to(user.username, url))
            return ", ".join(links)

        return ", ".join([user.username for user in asset.owners])

    def render_groups_for_grid(self, asset, field, value):
        asset_handler = self.app.get_asset_handler()
        groups = asset_handler.get_groups(asset)

        if self.farmos_style_grid_links:
            links = []
            for group in groups:
                url = self.request.route_url(
                    f"{group.asset_type}_assets.view", uuid=group.uuid
                )
                links.append(tags.link_to(str(group), url))
            return ", ".join(links)

        return ", ".join([str(group) for group in groups])

    def render_locations_for_grid(self, asset, field, value):
        asset_handler = self.app.get_asset_handler()
        locations = asset_handler.get_locations(asset)

        if self.farmos_style_grid_links:
            links = []
            for loc in locations:
                url = self.request.route_url(
                    f"{loc.asset_type}_assets.view", uuid=loc.uuid
                )
                links.append(tags.link_to(str(loc), url))
            return ", ".join(links)

        return ", ".join([str(loc) for loc in locations])

    def grid_row_class(self, asset, data, i):
        """ """
        if asset.archived:
            return "has-background-warning"
        return None

    def configure_form(self, form):
        f = form
        super().configure_form(f)
        asset_handler = self.app.get_asset_handler()
        asset = form.model_instance

        # asset_type
        if self.creating:
            f.remove("asset_type")
        else:
            f.set_node(
                "asset_type",
                WuttaDictEnum(self.request, get_asset_type_enum(self.config)),
            )
            f.set_readonly("asset_type")

        # owners
        if self.creating or self.editing:
            f.remove("owners")  # TODO: need to support this
        else:
            f.set_node("owners", OwnerRefs(self.request))
            # nb. must explicity declare value for non-standard field
            f.set_default("owners", asset.owners)

        # locations
        if self.creating or self.editing:
            # nb. this is a calculated field
            f.remove("locations")
        else:
            f.set_label("locations", "Current Location")
            f.set_node("locations", AssetRefs(self.request))
            # nb. must explicity declare value for non-standard field
            f.set_default("locations", asset_handler.get_locations(asset))

        # groups
        if self.creating or self.editing:
            # nb. this is a calculated field
            f.remove("groups")
        else:
            f.set_node("groups", AssetRefs(self.request))
            # nb. must explicity declare value for non-standard field
            f.set_default("groups", asset_handler.get_groups(asset))

        # parents
        f.set_node("parents", AssetRefs(self.request, for_asset=asset))
        f.set_required("parents", False)
        if not self.creating:
            # nb. must explicity declare value for non-standard field
            f.set_default("parents", asset.parents)

        # notes
        f.set_widget("notes", "notes")

        # thumbnail_url
        if self.creating or self.editing:
            f.remove("thumbnail_url")

        # image_url
        if self.creating or self.editing:
            f.remove("image_url")

        # thumbnail
        if self.creating or self.editing:
            f.remove("thumbnail")
        elif asset.thumbnail_url:
            f.set_widget("thumbnail", ImageWidget("animal thumbnail"))
            f.set_default("thumbnail", asset.thumbnail_url)

        # image
        if self.creating or self.editing:
            f.remove("image")
        elif asset.image_url:
            f.set_widget("image", ImageWidget("animal image"))
            f.set_default("image", asset.image_url)

    def objectify(self, form):
        model = self.app.model
        session = self.Session()
        asset = super().objectify(form)
        data = form.validated

        if self.creating:
            asset.asset_type = self.get_asset_type()

        current = [p.uuid for p in asset.parents]
        desired = data["parents"] or []

        for uuid in desired:
            if uuid not in current:
                parent = session.get(model.Asset, uuid)
                assert parent
                asset.parents.append(parent)

        for uuid in current:
            if uuid not in desired:
                parent = session.get(model.Asset, uuid)
                assert parent
                asset.parents.remove(parent)

        return asset

    def get_asset_type(self):
        model_class = self.get_model_class()
        return model_class.__wutta_hint__["farmos_asset_type"]

    def get_farmos_url(self, asset):
        return self.app.get_farmos_url(f"/asset/{asset.drupal_id}")

    def get_xref_buttons(self, asset):
        buttons = super().get_xref_buttons(asset)

        if asset.farmos_uuid:
            asset_type = self.get_asset_type()
            route = f"farmos_{asset_type}_assets.view"
            url = self.request.route_url(route, uuid=asset.farmos_uuid)
            buttons.append(
                self.make_button(
                    "View farmOS record", primary=True, url=url, icon_left="eye"
                )
            )

        return buttons

    def get_version_joins(self):
        """
        We override this to declare the relationship between the
        view's data model (which is some type of asset table) and the
        canonical ``Asset`` model, so the revision history views
        include transactions which reference either version table.

        See also parent method,
        :meth:`~wuttaweb:wuttaweb.views.master.MasterView.get_version_joins()`
        """
        model = self.app.model
        return super().get_version_joins() + [
            model.Asset,
        ]

    def get_row_grid_data(self, asset):
        model = self.app.model
        session = self.Session()
        return (
            session.query(model.Log)
            .outerjoin(model.LogAsset)
            .filter(model.LogAsset.asset_uuid == asset.uuid)
        )

    def configure_row_grid(self, grid):
        g = grid
        super().configure_row_grid(g)
        enum = self.app.enum
        model = self.app.model
        session = self.Session()

        # status
        g.set_enum("status", enum.LOG_STATUS)

        # drupal_id
        g.set_label("drupal_id", "ID", column_only=True)

        # message
        g.set_link("message")
        g.set_sorter("message", model.Log.message)
        g.set_filter("message", model.Log.message)

        # timestamp
        g.set_sorter("timestamp", model.Log.timestamp)
        g.set_filter("timestamp", model.Log.timestamp)

        # log_type
        g.set_sorter("log_type", model.Log.log_type)
        g.set_filter("log_type", model.Log.log_type)
        g.set_enum("log_type", get_log_type_enum(self.config, session=session))

        # assets
        g.set_renderer("assets", self.render_assets_for_grid)

    def render_assets_for_grid(self, log, field, value):
        assets = getattr(log, field)

        if self.farmos_style_grid_links:
            links = []
            for asset in assets:
                url = self.request.route_url(
                    f"{asset.asset_type}_assets.view", uuid=asset.uuid
                )
                links.append(tags.link_to(str(asset), url))
            return ", ".join(links)

        return ", ".join([str(a) for a in assets])

    def get_row_action_url_view(self, log, i):
        return self.request.route_url(f"logs_{log.log_type}.view", uuid=log.uuid)


class AllAssetView(AssetMasterView):
    """
    Master view for Assets
    """

    model_class = Asset
    route_prefix = "assets"
    url_prefix = "/assets"

    farmos_refurl_path = "/assets"

    viewable = False
    creatable = False
    editable = False
    deletable = False
    model_is_versioned = False

    grid_columns = [
        "thumbnail",
        "drupal_id",
        "asset_name",
        "groups",
        "asset_type",
        "parents",
        "owners",
        "locations",
        "archived",
    ]

    def configure_grid(self, grid):
        g = grid
        super().configure_grid(g)

        # asset_type
        g.set_enum("asset_type", get_asset_type_enum(self.config))

        # view action links to final asset record
        def asset_url(asset, i):
            return self.request.route_url(
                f"{asset.asset_type}_assets.view", uuid=asset.uuid
            )

        g.add_action("view", icon="eye", url=asset_url)


def defaults(config, **kwargs):
    base = globals()

    AllAssetView = kwargs.get("AllAssetView", base["AllAssetView"])
    AllAssetView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
WuttaFarm data models
"""

# bring in all of wutta
from wuttjamaican.db.model import *

# wutta model extensions
from .users import WuttaFarmUser

# wuttafarm proper models
from .unit import Unit, Measure
from .material_type import MaterialType
from .quantities import (
    QuantityType,
    Quantity,
    StandardQuantity,
    MaterialQuantity,
    MaterialQuantityMaterialType,
)
from .asset import AssetType, Asset, AssetParent
from .asset_land import LandType, LandAsset
from .asset_structure import StructureType, StructureAsset
from .asset_equipment import EquipmentType, EquipmentAsset, EquipmentAssetEquipmentType
from .asset_animal import AnimalType, AnimalAsset
from .asset_group import GroupAsset
from .asset_plant import (
    PlantType,
    Season,
    PlantAsset,
    PlantAssetPlantType,
    PlantAssetSeason,
)
from .asset_water import WaterAsset
from .log import LogType, Log, LogAsset, LogGroup, LogLocation, LogQuantity, LogOwner
from .log_activity import ActivityLog
from .log_harvest import HarvestLog
from .log_medical import MedicalLog
from .log_observation import ObservationLog
from .log_seeding import SeedingLog



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Custom views for Settings
"""

from webhelpers2.html import tags

from wuttaweb.views import settings as base

from wuttafarm.web.util import use_farmos_style_grid_links


class AppInfoView(base.AppInfoView):
    """
    Custom appinfo view
    """

    def get_appinfo_dict(self):
        info = super().get_appinfo_dict()
        enum = self.app.enum

        mode = self.config.get(
            f"{self.app.appname}.farmos_integration_mode", default="wrapper"
        )

        info["farmos_integration"] = {
            "label": "farmOS Integration",
            "value": enum.FARMOS_INTEGRATION_MODE.get(mode, mode),
        }

        url = self.app.get_farmos_url()
        info["farmos_url"] = {
            "label": "farmOS URL",
            "value": tags.link_to(url, url, target="_blank"),
        }

        return info

    def configure_get_simple_settings(self):  # pylint: disable=empty-docstring
        farmos = self.app.get_farmos_handler()
        simple_settings = super().configure_get_simple_settings()
        simple_settings.extend(
            [
                {
                    "name": "farmos.url.base",
                },
                {
                    "name": "farmos.oauth2.client_id",
                    "default": farmos.get_oauth2_client_id(),
                },
                {
                    "name": "farmos.oauth2.scope",
                    "default": farmos.get_oauth2_scope(),
                },
                {
                    "name": f"{self.app.appname}.farmos_integration_mode",
                    "default": self.app.get_farmos_integration_mode(),
                },
                {
                    "name": f"{self.app.appname}.farmos_style_grid_links",
                    "type": bool,
                    "default": use_farmos_style_grid_links(self.config),
                },
            ]
        )
        return simple_settings


def defaults(config, **kwargs):
    local = globals()
    AppInfoView = kwargs.get("AppInfoView", local["AppInfoView"])
    base.defaults(config, **{"AppInfoView": AppInfoView})


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Model definition for Equipment
"""

import sqlalchemy as sa
from sqlalchemy import orm
from sqlalchemy.ext.associationproxy import association_proxy

from wuttjamaican.db import model

from wuttafarm.db.model.taxonomy import TaxonomyMixin
from wuttafarm.db.model.asset import AssetMixin, add_asset_proxies


class EquipmentType(TaxonomyMixin, model.Base):
    """
    Represents an "equipment type" (taxonomy term) from farmOS
    """

    __tablename__ = "equipment_type"
    __versioned__ = {}
    __wutta_hint__ = {
        "model_title": "Equipment Type",
        "model_title_plural": "Equipment Types",
    }

    _equipment_assets = orm.relationship(
        "EquipmentAssetEquipmentType",
        cascade_backrefs=False,
        back_populates="equipment_type",
    )


class EquipmentAsset(AssetMixin, model.Base):
    """
    Represents an equipment asset from farmOS
    """

    __tablename__ = "asset_equipment"
    __versioned__ = {}
    __wutta_hint__ = {
        "model_title": "Equipment Asset",
        "model_title_plural": "Equipment Assets",
        "farmos_asset_type": "equipment",
    }

    manufacturer = sa.Column(
        sa.String(length=255),
        nullable=True,
        doc="""
        Name of the manufacturer, if applicable.
        """,
    )

    model = sa.Column(
        sa.String(length=255),
        nullable=True,
        doc="""
        Model name for the equipment, if applicable.
        """,
    )

    serial_number = sa.Column(
        sa.String(length=255),
        nullable=True,
        doc="""
        Serial number for the equipment, if applicable.
        """,
    )

    _equipment_types = orm.relationship(
        "EquipmentAssetEquipmentType",
        cascade="all, delete-orphan",
        cascade_backrefs=False,
        back_populates="equipment_asset",
    )

    equipment_types = association_proxy(
        "_equipment_types",
        "equipment_type",
        creator=lambda pt: EquipmentAssetEquipmentType(equipment_type=pt),
    )


add_asset_proxies(EquipmentAsset)


class EquipmentAssetEquipmentType(model.Base):
    """
    Associates one or more equipment types with an equipment asset.
    """

    __tablename__ = "asset_equipment_equipment_type"
    __versioned__ = {}

    uuid = model.uuid_column()

    equipment_asset_uuid = model.uuid_fk_column("asset_equipment.uuid", nullable=False)
    equipment_asset = orm.relationship(
        EquipmentAsset,
        foreign_keys=equipment_asset_uuid,
        back_populates="_equipment_types",
    )

    equipment_type_uuid = model.uuid_fk_column("equipment_type.uuid", nullable=False)
    equipment_type = orm.relationship(
        EquipmentType,
        doc="""
        Reference to the equipment type.
        """,
        back_populates="_equipment_assets",
    )



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Model definition for Plant Assets
"""

import sqlalchemy as sa
from sqlalchemy import orm
from sqlalchemy.ext.associationproxy import association_proxy

from wuttjamaican.db import model

from wuttafarm.db.model.asset import AssetMixin, add_asset_proxies


class PlantType(model.Base):
    """
    Represents a "plant type" (taxonomy term) from farmOS
    """

    __tablename__ = "plant_type"
    __versioned__ = {}
    __wutta_hint__ = {
        "model_title": "Plant Type",
        "model_title_plural": "Plant Types",
    }

    uuid = model.uuid_column()

    name = sa.Column(
        sa.String(length=100),
        nullable=False,
        unique=True,
        doc="""
        Name of the plant type.
        """,
    )

    description = sa.Column(
        sa.String(length=255),
        nullable=True,
        doc="""
        Optional description for the plant type.
        """,
    )

    farmos_uuid = sa.Column(
        model.UUID(),
        nullable=True,
        unique=True,
        doc="""
        UUID for the plant type within farmOS.
        """,
    )

    drupal_id = sa.Column(
        sa.Integer(),
        nullable=True,
        unique=True,
        doc="""
        Drupal internal ID for the plant type.
        """,
    )

    _plant_assets = orm.relationship(
        "PlantAssetPlantType",
        cascade_backrefs=False,
        back_populates="plant_type",
    )

    def __str__(self):
        return self.name or ""


class Season(model.Base):
    """
    Represents a "season" (taxonomy term) from farmOS
    """

    __tablename__ = "season"
    __versioned__ = {}
    __wutta_hint__ = {
        "model_title": "Season",
        "model_title_plural": "Seasons",
    }

    uuid = model.uuid_column()

    name = sa.Column(
        sa.String(length=100),
        nullable=False,
        unique=True,
        doc="""
        Name of the season.
        """,
    )

    description = sa.Column(
        sa.String(length=255),
        nullable=True,
        doc="""
        Optional description for the season.
        """,
    )

    farmos_uuid = sa.Column(
        model.UUID(),
        nullable=True,
        unique=True,
        doc="""
        UUID for the season within farmOS.
        """,
    )

    drupal_id = sa.Column(
        sa.Integer(),
        nullable=True,
        unique=True,
        doc="""
        Drupal internal ID for the season.
        """,
    )

    _plant_assets = orm.relationship(
        "PlantAssetSeason",
        cascade_backrefs=False,
        back_populates="season",
    )

    def __str__(self):
        return self.name or ""


class PlantAsset(AssetMixin, model.Base):
    """
    Represents a plant asset from farmOS
    """

    __tablename__ = "asset_plant"
    __versioned__ = {}
    __wutta_hint__ = {
        "model_title": "Plant Asset",
        "model_title_plural": "Plant Assets",
        "farmos_asset_type": "plant",
    }

    _plant_types = orm.relationship(
        "PlantAssetPlantType",
        cascade="all, delete-orphan",
        cascade_backrefs=False,
        back_populates="plant_asset",
    )

    plant_types = association_proxy(
        "_plant_types",
        "plant_type",
        creator=lambda pt: PlantAssetPlantType(plant_type=pt),
    )

    _seasons = orm.relationship(
        "PlantAssetSeason",
        cascade="all, delete-orphan",
        cascade_backrefs=False,
        back_populates="plant_asset",
    )

    seasons = association_proxy(
        "_seasons",
        "season",
        creator=lambda s: PlantAssetSeason(season=s),
    )


add_asset_proxies(PlantAsset)


class PlantAssetPlantType(model.Base):
    """
    Associates one or more plant types with a plant asset.
    """

    __tablename__ = "asset_plant_plant_type"
    __versioned__ = {}

    uuid = model.uuid_column()

    plant_asset_uuid = model.uuid_fk_column("asset_plant.uuid", nullable=False)
    plant_asset = orm.relationship(
        PlantAsset,
        foreign_keys=plant_asset_uuid,
        back_populates="_plant_types",
    )

    plant_type_uuid = model.uuid_fk_column("plant_type.uuid", nullable=False)
    plant_type = orm.relationship(
        PlantType,
        doc="""
        Reference to the plant type.
        """,
        back_populates="_plant_assets",
    )


class PlantAssetSeason(model.Base):
    """
    Associates one or more seasons with a plant asset.
    """

    __tablename__ = "asset_plant_season"
    __versioned__ = {}

    uuid = model.uuid_column()

    plant_asset_uuid = model.uuid_fk_column("asset_plant.uuid", nullable=False)
    plant_asset = orm.relationship(
        PlantAsset,
        foreign_keys=plant_asset_uuid,
        back_populates="_seasons",
    )

    season_uuid = model.uuid_fk_column("season.uuid", nullable=False)
    season = orm.relationship(
        Season,
        doc="""
        Reference to the season.
        """,
        back_populates="_plant_assets",
    )



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Base class for Quick Form views
"""

import logging

from pyramid.renderers import render_to_response

from wuttaweb.views import View
from wuttaweb.db import Session

from wuttafarm.web.util import get_farmos_client_for_user


log = logging.getLogger(__name__)


class QuickFormView(View):
    """
    Base class for quick form views.
    """

    Session = Session

    def __init__(self, request, context=None):
        super().__init__(request, context=context)
        self.farmos_client = get_farmos_client_for_user(self.request)
        self.farmos_4x = self.app.is_farmos_4x(self.farmos_client)
        self.normal = self.app.get_normalizer(self.farmos_client)

    @classmethod
    def get_route_slug(cls):
        return cls.route_slug

    @classmethod
    def get_url_slug(cls):
        return cls.url_slug

    @classmethod
    def get_form_title(cls):
        return cls.form_title

    def __call__(self):
        form = self.make_quick_form()

        if form.validate():
            try:
                result = self.save_quick_form(form)
            except Exception as err:
                log.warning("failed to save 'edit' form", exc_info=True)
                self.request.session.flash(
                    f"Save failed: {self.app.render_error(err)}", "error"
                )
            else:
                return self.redirect_after_save(result)

        return self.render_to_response({"form": form})

    def make_quick_form(self):
        raise NotImplementedError

    def save_quick_form(self, form):
        raise NotImplementedError

    def redirect_after_save(self, result):
        return self.redirect(self.request.current_route_url())

    def render_to_response(self, context):

        defaults = {
            "index_title": "Quick Form",
            "form_title": self.get_form_title(),
            "help_text": self.__doc__.strip(),
        }

        defaults.update(context)
        context = defaults

        # supplement context further if needed
        context = self.get_template_context(context)

        page_templates = self.get_page_templates()
        mako_path = page_templates[0]
        try:
            render_to_response(mako_path, context, request=self.request)
        except IOError:

            # try one or more fallback templates
            for fallback in page_templates[1:]:
                try:
                    return render_to_response(fallback, context, request=self.request)
                except IOError:
                    pass

            # if we made it all the way here, then we found no
            # templates at all, in which case re-attempt the first and
            # let that error raise on up
            return render_to_response(mako_path, context, request=self.request)

    def get_page_templates(self):
        route_slug = self.get_route_slug()
        page_templates = [f"/quick/{route_slug}.mako"]
        page_templates.extend(self.get_fallback_templates())
        return page_templates

    def get_fallback_templates(self):
        return ["/quick/form.mako"]

    def get_template_context(self, context):
        return context

    @classmethod
    def defaults(cls, config):
        cls._defaults(config)

    @classmethod
    def _defaults(cls, config):
        route_slug = cls.get_route_slug()
        url_slug = cls.get_url_slug()
        form_title = cls.get_form_title()

        config.add_wutta_permission("quick", f"quick.{route_slug}", form_title)
        config.add_route(f"quick.{route_slug}", f"/quick/{url_slug}")
        config.add_view(
            cls, route_name=f"quick.{route_slug}", permission=f"quick.{route_slug}"
        )



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Misc. utilities for web app
"""

from pyramid import httpexceptions
from webhelpers2.html import HTML


def get_farmos_client_for_user(request):
    token = request.session.get("farmos.oauth2.token")
    if not token:
        raise httpexceptions.HTTPForbidden()

    # nb. must give a *copy* of the token to farmOS client, since it
    # will mutate it in-place and we don't want that to happen for our
    # original copy in the user session.  (otherwise the auto-refresh
    # will not work correctly for subsequent calls.)
    token = dict(token)

    def token_updater(token):
        save_farmos_oauth2_token(request, token)

    config = request.wutta_config
    app = config.get_app()
    return app.get_farmos_client(token=token, token_updater=token_updater)


def save_farmos_oauth2_token(request, token):
    """
    Common logic for saving the given OAuth2 token within the user
    session.  This function is called from 2 places:

    * :meth:`wuttafarm.web.views.auth.farmos_oauth_callback()`
    * :meth:`wuttafarm.web.views.farmos.master.FarmOSMasterView.get_farmos_client()`
    """
    # nb. we pretend the token expires 1 minute early, to avoid edge
    # cases around token refresh
    token["expires_at"] -= 60

    # save token to user session
    request.session["farmos.oauth2.token"] = token


def use_farmos_style_grid_links(config):
    return config.get_bool(f"{config.appname}.farmos_style_grid_links", default=True)


def render_quantity_objects(quantities):
    items = []
    for quantity in quantities:
        text = render_quantity_object(quantity)
        items.append(HTML.tag("li", c=text))
    return HTML.tag("ul", c=items)


def render_quantity_object(quantity):
    measure = quantity["measure_name"]
    value = quantity["value_decimal"]
    unit = quantity["unit_name"]
    text = f"( {measure} ) {value} {unit}"

    if quantity["quantity_type_id"] == "material":
        materials = ", ".join(
            [mtype.get("name", "???") for mtype in quantity["material_types"]]
        )
        return f"{materials} {text}"

    return text



"""add LandAssetParent model

Revision ID: 554e6168c339
Revises: 8cc1565d38e7
Create Date: 2026-02-14 20:41:24.859064

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import wuttjamaican.db.util


# revision identifiers, used by Alembic.
revision: str = "554e6168c339"
down_revision: Union[str, None] = "8cc1565d38e7"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:

    # land_asset_parent
    op.create_table(
        "land_asset_parent",
        sa.Column("uuid", wuttjamaican.db.util.UUID(), nullable=False),
        sa.Column("land_asset_uuid", wuttjamaican.db.util.UUID(), nullable=False),
        sa.Column("parent_asset_uuid", wuttjamaican.db.util.UUID(), nullable=False),
        sa.ForeignKeyConstraint(
            ["land_asset_uuid"],
            ["land_asset.uuid"],
            name=op.f("fk_land_asset_parent_land_asset_uuid_land_asset"),
        ),
        sa.ForeignKeyConstraint(
            ["parent_asset_uuid"],
            ["land_asset.uuid"],
            name=op.f("fk_land_asset_parent_parent_asset_uuid_land_asset"),
        ),
        sa.PrimaryKeyConstraint("uuid", name=op.f("pk_land_asset_parent")),
    )
    op.create_table(
        "land_asset_parent_version",
        sa.Column(
            "uuid", wuttjamaican.db.util.UUID(), autoincrement=False, nullable=False
        ),
        sa.Column(
            "land_asset_uuid",
            wuttjamaican.db.util.UUID(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "parent_asset_uuid",
            wuttjamaican.db.util.UUID(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "transaction_id", sa.BigInteger(), autoincrement=False, nullable=False
        ),
        sa.Column("end_transaction_id", sa.BigInteger(), nullable=True),
        sa.Column("operation_type", sa.SmallInteger(), nullable=False),
        sa.PrimaryKeyConstraint(
            "uuid", "transaction_id", name=op.f("pk_land_asset_parent_version")
        ),
    )
    op.create_index(
        op.f("ix_land_asset_parent_version_end_transaction_id"),
        "land_asset_parent_version",
        ["end_transaction_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_land_asset_parent_version_operation_type"),
        "land_asset_parent_version",
        ["operation_type"],
        unique=False,
    )
    op.create_index(
        "ix_land_asset_parent_version_pk_transaction_id",
        "land_asset_parent_version",
        ["uuid", sa.literal_column("transaction_id DESC")],
        unique=False,
    )
    op.create_index(
        "ix_land_asset_parent_version_pk_validity",
        "land_asset_parent_version",
        ["uuid", "transaction_id", "end_transaction_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_land_asset_parent_version_transaction_id"),
        "land_asset_parent_version",
        ["transaction_id"],
        unique=False,
    )


def downgrade() -> None:

    # land_asset_parent
    op.drop_index(
        op.f("ix_land_asset_parent_version_transaction_id"),
        table_name="land_asset_parent_version",
    )
    op.drop_index(
        "ix_land_asset_parent_version_pk_validity",
        table_name="land_asset_parent_version",
    )
    op.drop_index(
        "ix_land_asset_parent_version_pk_transaction_id",
        table_name="land_asset_parent_version",
    )
    op.drop_index(
        op.f("ix_land_asset_parent_version_operation_type"),
        table_name="land_asset_parent_version",
    )
    op.drop_index(
        op.f("ix_land_asset_parent_version_end_transaction_id"),
        table_name="land_asset_parent_version",
    )
    op.drop_table("land_asset_parent_version")
    op.drop_table("land_asset_parent")



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
WuttaFarm CLI
"""

from .base import wuttafarm_typer

# nb. must bring in all modules for discovery to work
from . import export_farmos
from . import import_farmos
from . import install



"""add Log.is_movement

Revision ID: 0771322957bd
Revises: 12de43facb95
Create Date: 2026-03-02 20:21:03.889847

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import wuttjamaican.db.util


# revision identifiers, used by Alembic.
revision: str = "0771322957bd"
down_revision: Union[str, None] = "12de43facb95"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:

    # log
    op.add_column("log", sa.Column("is_movement", sa.Boolean(), nullable=True))
    op.add_column(
        "log_version",
        sa.Column("is_movement", sa.Boolean(), autoincrement=False, nullable=True),
    )


def downgrade() -> None:

    # log
    op.drop_column("log_version", "is_movement")
    op.drop_column("log", "is_movement")



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Model definition for Seeding Logs
"""

import sqlalchemy as sa

from wuttjamaican.db import model

from wuttafarm.db.model.log import LogMixin, add_log_proxies


class SeedingLog(LogMixin, model.Base):
    """
    Represents a Seeding Log from farmOS
    """

    __tablename__ = "log_seeding"
    __versioned__ = {}
    __wutta_hint__ = {
        "model_title": "Seeding Log",
        "model_title_plural": "Seeding Logs",
        "farmos_log_type": "seeding",
    }

    source = sa.Column(
        sa.String(length=255),
        nullable=True,
        doc="""
        Where the seed was obtained, if applicable.
        """,
    )

    purchase_date = sa.Column(
        sa.DateTime(),
        nullable=True,
        doc="""
        When the seed was purchased, if applicable.
        """,
    )

    lot_number = sa.Column(
        sa.String(length=255),
        nullable=True,
        doc="""
        Lot number for the seed, if applicable.
        """,
    )


add_log_proxies(SeedingLog)



"""add Log.is_group_assignment

Revision ID: f3c7e273bfa3
Revises: 47d0ebd84554
Create Date: 2026-02-28 20:04:40.700474

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import wuttjamaican.db.util


# revision identifiers, used by Alembic.
revision: str = "f3c7e273bfa3"
down_revision: Union[str, None] = "47d0ebd84554"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:

    # log
    op.add_column("log", sa.Column("is_group_assignment", sa.Boolean(), nullable=True))
    op.add_column(
        "log_version",
        sa.Column(
            "is_group_assignment", sa.Boolean(), autoincrement=False, nullable=True
        ),
    )


def downgrade() -> None:

    # log
    op.drop_column("log_version", "is_group_assignment")
    op.drop_column("log", "is_group_assignment")



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Master view for Water Assets in farmOS
"""

from wuttafarm.web.views.farmos.assets import AssetMasterView


class WaterAssetView(AssetMasterView):
    """
    Master view for farmOS Water Assets
    """

    model_name = "farmos_water_assets"
    model_title = "farmOS Water Asset"
    model_title_plural = "farmOS Water Assets"

    route_prefix = "farmos_water_assets"
    url_prefix = "/farmOS/assets/water"

    farmos_asset_type = "water"
    farmos_refurl_path = "/assets/water"

    grid_columns = [
        "thumbnail",
        "drupal_id",
        "name",
        "archived",
    ]

    def get_xref_buttons(self, water):
        model = self.app.model
        session = self.Session()
        buttons = super().get_xref_buttons(water)

        if wf_water := (
            session.query(model.Asset)
            .filter(model.Asset.farmos_uuid == water["uuid"])
            .first()
        ):
            buttons.append(
                self.make_button(
                    f"View {self.app.get_title()} record",
                    primary=True,
                    url=self.request.route_url("water_assets.view", uuid=wf_water.uuid),
                    icon_left="eye",
                )
            )

        return buttons


def defaults(config, **kwargs):
    base = globals()

    WaterAssetView = kwargs.get("WaterAssetView", base["WaterAssetView"])
    WaterAssetView.defaults(config)


def includeme(config):
    defaults(config)



"""add Log.quick

Revision ID: 85d4851e8292
Revises: d459db991404
Create Date: 2026-03-02 18:42:56.070281

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
import wuttjamaican.db.util


# revision identifiers, used by Alembic.
revision: str = "85d4851e8292"
down_revision: Union[str, None] = "d459db991404"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:

    # log
    op.add_column("log", sa.Column("quick", sa.String(length=20), nullable=True))
    op.add_column(
        "log_version",
        sa.Column("quick", sa.String(length=20), autoincrement=False, nullable=True),
    )


def downgrade() -> None:

    # log
    op.drop_column("log_version", "quick")
    op.drop_column("log", "quick")



# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaFarm --Web app to integrate with and extend farmOS
#  Copyright © 2026 Lance Edgar
#
#  This file is part of WuttaFarm.
#
#  WuttaFarm is free software: you can redistribute it and/or modify it under
#  the terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  WuttaFarm is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along with
#  WuttaFarm.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
WuttaFarm Views
"""

from wuttaweb.views import essential

from .master import WuttaFarmMasterView, TaxonomyMasterView


def includeme(config):
    wutta_config = config.registry.settings.get("wutta_config")
    app = wutta_config.get_app()
    enum = app.enum
    mode = app.get_farmos_integration_mode()

    # wuttaweb core
    essential.defaults(
        config,
        **{
            "wuttaweb.views.auth": "wuttafarm.web.views.auth",
            "wuttaweb.views.common": "wuttafarm.web.views.common",
            "wuttaweb.views.settings": "wuttafarm.web.views.settings",
            "wuttaweb.views.users": "wuttafarm.web.views.users",
        }
    )

    # native table views
    if mode != enum.FARMOS_INTEGRATION_MODE_WRAPPER:
        config.include("wuttafarm.web.views.units")
        config.include("wuttafarm.web.views.material_types")
        config.include("wuttafarm.web.views.quantities")
        config.include("wuttafarm.web.views.asset_types")
        config.include("wuttafarm.web.views.assets")
        config.include("wuttafarm.web.views.land")
        config.include("wuttafarm.web.views.structures")
        config.include("wuttafarm.web.views.equipment")
        config.include("wuttafarm.web.views.animals")
        config.include("wuttafarm.web.views.groups")
        config.include("wuttafarm.web.views.plants")
        config.include("wuttafarm.web.views.water")
        config.include("wuttafarm.web.views.logs")
        config.include("wuttafarm.web.views.logs_activity")
        config.include("wuttafarm.web.views.logs_harvest")
        config.include("wuttafarm.web.views.logs_medical")
        config.include("wuttafarm.web.views.logs_observation")
        config.include("wuttafarm.web.views.logs_seeding")

    # quick form views
    # (nb. these work with all integration modes)
    config.include("wuttafarm.web.views.quick")

    # views for farmOS
    if mode != enum.FARMOS_INTEGRATION_MODE_NONE:
        config.include("wuttafarm.web.views.farmos")
